package de.intevation.flys.artifacts.model;

import java.util.List;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;

import org.hibernate.Session;

import org.hibernate.SQLQuery;
import org.hibernate.type.StandardBasicTypes;

import de.intevation.flys.artifacts.model.WKms;
import de.intevation.flys.artifacts.model.WKmsImpl;

import de.intevation.flys.artifacts.cache.CacheFactory;

import de.intevation.flys.backend.SessionHolder;

/**
 * Factory to access ready-made WKms for other (than computed) 'kinds' of 
 * WST-data.
 */
public class WKmsFactory
{
    private static Logger log = Logger.getLogger(WKmsFactory.class);

    /**
     *  Value for wsts 'kind' typically used for "additional
     *  longitudinal sections.
     */
    public static final int KIND_ZUS = 1;

    /**
     * Value for wsts 'kind' typically used for flood protection measures
     * (e.g. dikes).
     */
    public static final int KIND_PROTECTION = 5;

    /** Query to get km and ws for wst_id and column_pos. */
    public static final String SQL_SELECT_WS =
        "SELECT km, w FROM wst_w_values " +
        "WHERE wst_id = :wst_id AND column_pos = :column_pos";


    private WKmsFactory() {
    }


    /**
     * Get WKms for given column and wst_id, caring about the cache.
     */
    public static WKms getWKms(int kind, int column, int wst_id) {
        log.debug("WKmsFactory.getWKms");
        Cache cache = CacheFactory.getCache(StaticWKmsCacheKey.CACHE_NAME);

        StaticWKmsCacheKey cacheKey;

        if (cache != null) {
            cacheKey = new StaticWKmsCacheKey(wst_id, column);
            Element element = cache.get(cacheKey);
            if (element != null) {
                log.debug("Got static wst values from cache");
                return (WKms)element.getValue();
            }
        }
        else {
            cacheKey = null;
        }

        WKms values = getWKmsUncached(kind, column, wst_id);

        if (values != null && cacheKey != null) {
            log.debug("Store static wst values in cache.");
            Element element = new Element(cacheKey, values);
            cache.put(element);
        }
        return values;
    }


    /**
     * Get WKms from db.
     */
    public static WKms getWKmsUncached(int kind, int column, int wst_id) {
        log.debug("WKmsFactory.getWKmsUncached c/" + column + ", wst_id/" + wst_id);
        Session session = SessionHolder.HOLDER.get();

        SQLQuery sqlQuery = session.createSQLQuery(SQL_SELECT_WS)
            .addScalar("km", StandardBasicTypes.DOUBLE)
            .addScalar("w",  StandardBasicTypes.DOUBLE);
        sqlQuery.setInteger("wst_id",     wst_id);
        sqlQuery.setInteger("column_pos", column);

        List<Object []> results = sqlQuery.list();

        // TODO Fetch name in a separate query.
        String name = "todo";

        WKmsImpl wkms = new WKmsImpl(name);
        double kms [] = new double[results.size()];
        double ws  [] = new double[results.size()];

        int lastColumn = Integer.MAX_VALUE;

        for (int i = 0; i < results.size(); i++) {
            Object[] row = results.get(i);
            wkms.add((Double) row[0], (Double) row[1]);
        }

        return wkms;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
